clear,clc

%Load data
path = '/media/lukas/F/bachelor_thesis-20190522T104024Z-002/bachelor_thesis/mobile_robot_dataset1/measure301/';
data_odometry = dlmread(strcat(path,'odometry.csv'),',');
data_slam = dlmread(strcat(path , 'ORB_OUTPUT_MATLAB.csv'));
data_ground = dlmread(strcat(path , 'ground_truth_matlab.csv'));
use_plot= true;

%Modify odometry
odo_plot = data_odometry(:,1:4);
data_odometry = [data_odometry(:,1) data_odometry(:,5:6)];
data_odometry(:,2) = data_odometry(:,2) - data_odometry(1,2);
data_odometry(:,3) = data_odometry(:,3) - data_odometry(1,3);
data_odometry(2:end,2) = diff(data_odometry(:,2));
data_odometry(2:end,3) = diff(data_odometry(:,3));

% Compensate sensor offset for SLAM
ampl = -0.15;
alpha_off = 0.4;
for i = 1:size(data_slam,1)
    temp = trans(0,0,deg2rad(-90))*[data_slam(i,2);data_slam(i,3);1];
    data_slam(i,2) = temp(1)/temp(3)+(ampl*cos(data_slam(i,4)+alpha_off)+0.15);
    data_slam(i,3) = temp(2)/temp(3)+(ampl*sin(data_slam(i,4)+alpha_off)+0.05);
   if data_slam(i,4) < -1e-3
       data_slam(i,4) = deg2rad(180) + deg2rad(180) + data_slam(i,4);
   end
end

% Compensate sensor offset ground for groundtruth
ampl = -0.15;
alpha_off = 0.4;
for i = 1:size(data_ground,1)
    temp = trans(0,0,deg2rad(-90))*[data_ground(i,2);data_ground(i,3);1];
    data_ground(i,2) = temp(1)/temp(3)+(ampl*cos(data_ground(i,4)+alpha_off)+0.15);
    data_ground(i,3) = temp(2)/temp(3)+(ampl*sin(data_ground(i,4)+alpha_off)+0.05);
end

%Mark data
data_odometry = [data_odometry zeros(size(data_odometry,1),2)];
data_slam = [data_slam ones(size(data_slam,1),1)];

%Sort data acccording to time
kalman_data = [data_odometry;
    data_slam];
kalman_data = sortrows(kalman_data,1);

%Intitialize EKF
last_pos = [0;0;0];

Q = [0.01 0;
    0 0.01];

R = [1 0 0;
    0 1 0;
    0 0 0.5];

P = [0.1 0 0;
    0 0.1 0;
    0 0 0.1];

kalman_plot = [kalman_data(1,1)' last_pos'];
tic
%Kalman filter
 for i=1:size(kalman_data,1)
     %Predict step
    if kalman_data(i,5)==0          
        %Update estimate
        [x,P_new] = predict(last_pos,kalman_data(i,2:3)',P,Q);
        P=P_new;
        last_pos = x;
        
        %Add to plot
        if use_plot
            kalman_plot = [kalman_plot; kalman_data(i,1)' last_pos'];
        end
    else
        %Update step   
        %Update estimate
        [x,P_new] = update(kalman_data(i,2:4)',last_pos,last_pos,P,R);
        P=P_new;
        last_pos = x;        
        %Add to plot
        if use_plot
            kalman_plot = [kalman_plot; kalman_data(i,1)' last_pos'];
        end
    end  
 end
 toc
 dist_t = sqrt((data_ground(end,2)- kalman_plot(end,2))^2 +  (data_ground(end,3)- kalman_plot(end,3))^2);
 strcat('Trans error:',{' '},string(dist_t)) 
 
norm_angle = atan2(sin(kalman_plot(end,4)),cos(kalman_plot(end,4)));
dist_a =rad2deg(sqrt((data_ground(end,4)- norm_angle)^2));
strcat('Ang error:',{' '},string(dist_a)) 
 
function A = trans(x_t,y_t,alpha)
    A = [cos(alpha) sin(alpha) x_t;
        -sin(alpha) cos(alpha) y_t;
        0 0 1 ];
end