function [x,P] = predict(x_t,u_t,P_t,Q)
%PREDICT Predict stage of EKF
%   Function computes prediction based on current states x_t,
%   current control input u_t and state covariance matrix P and 
%   input covariance Q
     
delta_s = (u_t(1)+u_t(2))/2;
delta_omega = (-u_t(1)+u_t(2))/1.0447;

x(1,1) = x_t(1,1) + delta_s*cos(x_t(3,1)+delta_omega/2);
x(2,1) = x_t(2,1) + delta_s*sin(x_t(3,1)+delta_omega/2);
x(3,1) = x_t(3,1) + delta_omega;

F= [ 1, 0, -delta_s*sin(x_t(3,1)+delta_omega/2);
    0,  1, delta_s*cos(x_t(3,1)+delta_omega/2);
    0, 0, 1];

alpha_last = x_t(3,1);
d_l = u_t(2);
d_r = u_t(1);

G =[ cos(alpha_last + d_l/4 - d_r/4)/2 + sin(alpha_last + d_l/4 - d_r/4)*(d_l/8 + d_r/8), cos(alpha_last + d_l/4 - d_r/4)/2 - (sin(alpha_last + d_l/4 - d_r/4)*(d_l/2 + d_r/2))/4;
 sin(alpha_last + d_l/4 - d_r/4)/2 - (cos(alpha_last + d_l/4 - d_r/4)*(d_l/2 + d_r/2))/4,     sin(alpha_last + d_l/4 - d_r/4)/2 + cos(alpha_last + d_l/4 - d_r/4)*(d_l/8 + d_r/8);
-0.5, 0.5];

 P = F*P_t*F'+G*Q*G';
end

